#!/usr/bin/env python
# encoding: utf-8
"""
This package provided some utils and tools for developer.
The difference between utils and tools is that tool can
use other utils module is this package but util did not.
It only allow use standard libraries whatever utils or
tools.Remember named a util module ends with utils.
"""
